const Lang = imports.lang;
const St = imports.gi.St;

const ExtensionUtils = imports.misc.extensionUtils;
const Main = imports.ui.main;
const PointerWatcher = imports.ui.pointerWatcher;
const AppDisplay = imports.ui.appDisplay;

const MOUSE_POLL_FREQUENCY = 10;

let data = {};

function applyImprovements(apply) {
	let allView = Main.overview.viewSelector.appDisplay._views[AppDisplay.Views['ALL']].view;
	let stylesheet = ExtensionUtils.getCurrentExtension().dir.get_child('fade.css');
	let theme = St.ThemeContext.get_for_stage(global.stage).get_theme();

	if (apply) {
		data['addWatch'] = PointerWatcher.PointerWatcher.prototype.addWatch;
		PointerWatcher.PointerWatcher.prototype.addWatch = function (interval, callback) {
			return data['addWatch'].call(this, MOUSE_POLL_FREQUENCY, callback);
		}

		data['Allview.adaptToSize'] = AppDisplay.AllView.prototype.adaptToSize;
		AppDisplay.AllView.prototype.adaptToSize = function (width, height) {
			data['Allview.adaptToSize'].call(this, width, height);

			this._scrollView.update_fade_effect(0, 0);
		}

		data['FolderView.adaptToSize'] = AppDisplay.FolderView.prototype.adaptToSize;
		AppDisplay.FolderView.prototype.adaptToSize = function (width, height) {
			data['FolderView.adaptToSize'].call(this, width, height);

			this.actor.update_fade_effect(0, 0);
		}

		data['_closeSpaceForPopup'] = AppDisplay.AllView.prototype._closeSpaceForPopup;
		AppDisplay.AllView.prototype._closeSpaceForPopup = function () {
			this._updateIconOpacities(false);
	        this._grid.closeExtraSpace();
		}

		data['space-opened'] = allView._grid.connect('space-opened', Lang.bind(allView, function() {
			this.emit('space-ready');
		}));

		theme.load_stylesheet(stylesheet);
	}
	else {
		if (data['space-opened']) {
			allView._grid.disconnect(data['space-opened']);
			delete data['space-opened'];
		}

		PointerWatcher.PointerWatcher.prototype.addWatch = data['addWatch'];
		AppDisplay.AllView.prototype.adaptToSize = data['Allview.adaptToSize'];
		AppDisplay.FolderView.prototype.adaptToSize = data['FolderView.adaptToSize'];
		AppDisplay.AllView.prototype._closeSpaceForPopup = data['_closeSpaceForPopup'];

		theme.unload_stylesheet(stylesheet);
	}

	Main.magnifier.stopTrackingMouse();
	Main.magnifier.startTrackingMouse();

	Main.loadTheme();

	allView._scrollView.queue_redraw();
}

function enable() {
	if (Main.magnifier.isActive())
		applyImprovements(true);

	data['active-changed'] = Main.magnifier.connect('active-changed', function() {
		applyImprovements(Main.magnifier.isActive());
	});
}

function disable() {
	Main.magnifier.disconnect(data['active-changed']);

	applyImprovements(false);
}

function init() {}